package com.fjhx.rfid.r2000.udp.service.impl;

import com.fjhx.config.RFIDConfig;
import com.fjhx.constant.RFIDConstant;
import com.fjhx.rfid.r2000.udp.callback.SdUdpCallBack;
import com.fjhx.rfid.r2000.udp.service.ISdUdpService;
import com.fjhx.rfid.r2000.udp.utils.Utility;
import com.fjhx.thread.ThreadPoolManager;
import com.fjhx.utils.AjaxResult;
import com.fjhx.web.exception.BaseException;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.DatagramPacket;
import io.netty.channel.socket.nio.NioDatagramChannel;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;

import java.net.InetSocketAddress;
import java.text.MessageFormat;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CountDownLatch;

/**
 * @Description:
 * @ClassName: UdpServiceImpl
 * @Author: linqt
 * @Date: 2022/3/18 11:22
 * @Version: 1.0
 */
@Service
public class SdUdpServiceImpl implements ISdUdpService {

    /**
     * 初始化实例集合
     */
    private static ConcurrentHashMap<String, Object> sdUdpServerMap = new ConcurrentHashMap<>();

    /**
     * 启动读写器
     *
     * @param ips 读写器ip集合
     * @return
     */
    @Override
    public AjaxResult open(List<String> ips) {
        if (CollectionUtils.isEmpty(ips)) {
            throw new BaseException("读写器IP地址不能为空！");
        }
        try {
            CountDownLatch countDownLatch = new CountDownLatch(ips.size());
            // 启动扫描线程
            for (String ip : ips) {
                //如果读写器已开启，则跳过
                if (ObjectUtils.isNotEmpty(sdUdpServerMap.get(ip))) {
                    System.err.println("IP为【" + ip + "】的读写器已开启。");
                    continue;
                }

                CountDownLatch errCountDownLatch = countDownLatch;
                ThreadPoolManager.execute(() -> {
                    try {
                        EventLoopGroup group = new NioEventLoopGroup();
                        Bootstrap b = new Bootstrap();
                        b.group(group)
                                .channel(NioDatagramChannel.class)
                                .option(ChannelOption.SO_BROADCAST, true)
                                .option(ChannelOption.ALLOCATOR, PooledByteBufAllocator.DEFAULT)
                                .handler(new SdUdpCallBack(ip));
                        Channel ch = b.bind(RFIDConstant.RFID_UDP_DEFAULT_BINDING_PORT).sync().channel();
                        // 读取
                        byte[] sendData = Utility.convert2HexArray(RFIDConstant.RFID_UDP_START_READ_INSTRUCTION);
                        ByteBuf byteBuf = Unpooled.copiedBuffer(sendData);
                        ch.writeAndFlush(new DatagramPacket(byteBuf, new InetSocketAddress(ip, RFIDConfig.getSd().getUdp().getPort()))).sync();

                        // 实例静态化
                        sdUdpServerMap.put(ip, ch);
                    } catch (Exception e) {
                        e.printStackTrace();
                        if (errCountDownLatch != null) {
                            errCountDownLatch.countDown();
                        }
                        throw new BaseException(MessageFormat.format(RFIDConstant.SERVER_START_ERR, ip));
                    }
                });
            }

            if (countDownLatch != null) {
                countDownLatch.await();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.success();
    }

    /**
     * 关闭读写器
     *
     * @param ips 读写器ip集合
     * @return
     */
    @Override
    public AjaxResult close(List<String> ips) {
        if (CollectionUtils.isEmpty(ips)) {
            throw new BaseException("参数缺失！");
        }
        // 关闭线程
        for (String ip : ips) {
            try {
                //如果读写器已开启，则跳过
                if (ObjectUtils.isEmpty(sdUdpServerMap.get(ip))) {
                    System.err.println("IP为【" + ip + "】的读写器已关闭。");
                    continue;
                }

                // 获取实例
                Channel ch = (Channel) sdUdpServerMap.get(ip);
                // 停止读取
                byte[] sendData = Utility.convert2HexArray(RFIDConstant.RFID_UDP_STOP_READ_INSTRUCTION);
                ByteBuf byteBuf = Unpooled.copiedBuffer(sendData);
                ch.writeAndFlush(new DatagramPacket(byteBuf, new InetSocketAddress(ip, RFIDConfig.getSd().getUdp().getPort()))).sync();
                ch.close();

                // 移除服务
                sdUdpServerMap.remove(ip);
            } catch (Exception e) {
                e.printStackTrace();
                throw new BaseException(MessageFormat.format(RFIDConstant.SERVER_CLOSE_ERR, ip));
            }
        }

        return AjaxResult.success();
    }
}
